# Copyright 2007-2015 UShareSoft SAS, All rights reserved
#
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os
import os.path
import ntpath
from ussclicore.utils import generics_utils, printer
from hammr.utils import constants
from uforge.objects.uforge import *

def check_bundle(bundle):
    if not "name" in bundle:
        printer.out("There is no attribute [name] for a [bundle]", printer.ERROR)
        return
    if not "version" in bundle:
        printer.out("no attribute [version] for [bundle]", printer.ERROR)
        return
    if not "files" in bundle:
        printer.out("no attribute [files] for [bundle]", printer.ERROR)
        return

    for file in bundle["files"]:
        bundle = check_files(bundle, file, 0)
        if bundle is None:
            return

    return bundle

def check_files(bundle, file, level):
    if not "name" in file:
        printer.out("There is no attribute [name] for a [file]", printer.ERROR)
        return
    if not "source" in file:
        printer.out("There is no attribute [source] for a [file]", printer.ERROR)
        return

    if level > 0 and "tag" in file and (file["tag"] == "ospkg" or file["tag"] == "bootscript"):
        printer.out("The file '" + file["name"] + ", with tag '" + file["tag"] + "' must be in the first level files section", printer.ERROR)
        return
    if ("bootOrder" in file or "bootType" in file) and (not "tag" in file or file["tag"] != "bootscript"):
        printer.out("There is the attribute [bootOrder] or [bootType] for file '" + file["name"] + "' but is not tagged as 'bootscript'", printer.ERROR)
        return
    if ("ownerGroup" in file or "rights" in file or "symlink" in file) and "tag" in file and file["tag"] != "softwarefile":
        printer.out("There is the attribute [ownerGroup], [rights] or [symlink] for file '" + file["name"] + "' but is not tagged as 'softwarefile'", printer.ERROR)
        return

    if "files" in file:
        if len(file["files"]) > 0:
            if "tag" in file and file["tag"] != "softwarefile":
                printer.out("The list of files is not empty for file '" + file["name"] + "' but is not tagged as 'softwarefile'", printer.ERROR)
                return
            elif not os.path.isdir(file["source"]):
                printer.out("The list of files is not empty for file '" + file["name"] + "' but [source] doesn't represent a folder", printer.ERROR)
                return

            for subFile in file["files"]:
                bundle = check_files(bundle, subFile, level + 1)

    return bundle

def recursivelyAppendToArchive(bundle, files, parentDir, checkList, archive_files):
    #must save the filepath before changing it after archive
    filePathBeforeTar = files["source"]
    if not "tag" in files or ("tag" in files and files["tag"] != "ospkg"):
        if files["source"] not in checkList:
            #add the source path to the check list
            checkList.append(files["source"])
            #if parentDir is a no empty path, add os.sep after. Else keep it as ""
            if parentDir:
                parentDir = parentDir + os.sep

            #add to list of file to tar
            file_tar_path=constants.FOLDER_BUNDLES + os.sep + generics_utils.remove_URI_forbidden_char(bundle["name"]) + os.sep + generics_utils.remove_URI_forbidden_char(bundle["version"]) + os.sep + parentDir + generics_utils.remove_URI_forbidden_char(ntpath.basename(files["source"]))
            archive_files.append([file_tar_path,files["source"]])
            #changing source path to archive related source path
            files["source"]=file_tar_path
        else:
            printer.out("found two files with the same source path in the bundles section...", printer.ERROR)
            return 2

    if "files" in files:
        for subFiles in files["files"]:
            checkList,archive_files = recursivelyAppendToArchive(bundle, subFiles, parentDir + ntpath.basename(files["source"]), checkList, archive_files)

    if (not "tag" in files or files["tag"] != "ospkg") and os.path.isdir(filePathBeforeTar):
        checkList,archive_files = processFilesFromFolder(bundle, files, filePathBeforeTar, parentDir + ntpath.basename(filePathBeforeTar), checkList, archive_files)

    return checkList, archive_files

def processFilesFromFolder(bundle, files, filePath, parentDir, checkList, archive_files):
    for subFiles in os.listdir(filePath):
        subFilesDict = dict({"name" : ntpath.basename(subFiles), "source" : filePath + os.sep + ntpath.basename(subFiles), "files" : []})
        #must save the filepath before changing it after archive
        subFilePathBeforeTar = subFilesDict["source"]
        if subFilesDict["source"] not in checkList:
            #add the source path to the check list
            checkList.append(subFilesDict["source"])
            #if parentDir is a no empty path, add os.sep after. Else keep it as ""
            if parentDir:
                parentDir = parentDir + os.sep

            #add to list of file to tar
            file_tar_path=constants.FOLDER_BUNDLES + os.sep + generics_utils.remove_URI_forbidden_char(bundle["name"]) + os.sep + generics_utils.remove_URI_forbidden_char(bundle["version"]) + os.sep + parentDir + generics_utils.remove_URI_forbidden_char(ntpath.basename(subFilesDict["source"]))
            archive_files.append([file_tar_path,subFilesDict["source"]])
            #changing source path to archive related source path and add it to files section of parent folder
            subFilesDict["source"] = file_tar_path
            files["files"].append(subFilesDict)

        if os.path.isdir(subFilePathBeforeTar):
            checkList,archive_files = processFilesFromFolder(bundle, subFilesDict, subFilePathBeforeTar, parentDir + ntpath.basename(subFilePathBeforeTar), checkList, archive_files)

    return checkList, archive_files


# Copyright 2007-2015 UShareSoft SAS, All rights reserved
#
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import ntpath

from uforge.objects.uforge import *
from ussclicore.utils import generics_utils, printer
from pyxb.utils import domutils


def openstack(account):
    myCredAccount = CredAccountOpenStack()
    # doing field verification
    if not "name" in account:
        printer.out("name for openstack account not found", printer.ERROR)
        return
    if not "glanceUrl" in account:
        printer.out("glanceUrl for openstack account not found", printer.ERROR)
        return
    if not "keystoneUrl" in account:
        printer.out("KeystoneUrl for openstack account not found", printer.ERROR)
        return
    if not "keystoneVersion" in account:
        printer.out("keystoneVersion for openstack account not found", printer.ERROR)
        return
    if not "login" in account:
        printer.out("login in openstack account not found", printer.ERROR)
        return
    if not "password" in account:
        printer.out("password in openstack account not found", printer.ERROR)
        return

    myCredAccount.name = account["name"]
    myCredAccount.glanceUrl = account["glanceUrl"]
    myCredAccount.keystoneUrl = account["keystoneUrl"]
    myCredAccount.keystoneVersion = account["keystoneVersion"]
    myCredAccount.login = account["login"]
    myCredAccount.password = account["password"]
    return myCredAccount


def susecloud(account):
    myCredAccount = SuseCloud()
    # doing field verification
    if not "username" in account:
        printer.out("username in susecloud account not found", printer.ERROR)
        return
    if not "password" in account:
        printer.out("catalogName in susecloud account not found", printer.ERROR)
        return
    if not "endpoint" in account:
        printer.out("endpoint for susecloud account not found", printer.ERROR)
        return
    if not "keystoneEndpoint" in account:
        printer.out("keystoneEndpoint for susecloud account not found", printer.ERROR)
        return
    if not "name" in account:
        printer.out("name for susecloud account not found", printer.ERROR)
        return

    myCredAccount.login = account["username"]
    myCredAccount.password = account["password"]
    myCredAccount.serverUrl = account["endpoint"]
    myCredAccount.keystoneUrl = account["keystoneEndpoint"]
    myCredAccount.name = account["name"]
    return myCredAccount


def cloudstack(account):
    myCredAccount = CredAccountCloudStack()
    # doing field verification
    if not "name" in account:
        printer.out("name for cloudstack account not found", printer.ERROR)
        return
    if not "publicApiKey" in account:
        printer.out("publicApiKey in cloudstack account not found", printer.ERROR)
        return
    if not "secretApiKey" in account:
        printer.out("secretApiKey in cloudstack account not found", printer.ERROR)
        return
    if not "endpointUrl" in account:
        printer.out("endpointUrl for cloudstack account not found", printer.ERROR)
        return

    myCredAccount.name = account["name"]
    myCredAccount.publicApiKey = account["publicApiKey"]
    myCredAccount.secretApiKey = account["secretApiKey"]
    myCredAccount.endpointUrl = account["endpointUrl"]
    return myCredAccount


def aws(account):
    myCredAccount = CredAccountAws()
    # doing field verification
    if not "accountNumber" in account:
        printer.out("accountNumber for ami account not found", printer.ERROR)
        return
    if not "name" in account:
        printer.out("name for ami account not found", printer.ERROR)
        return
    if not "accessKeyId" in account:
        printer.out("accessKey in ami account not found", printer.ERROR)
        return
    if not "secretAccessKeyId" in account:
        printer.out("secretAccessKey in ami account not found", printer.ERROR)
        return
    if not "x509Cert" in account:
        printer.out("x509Cert in ami account not found", printer.ERROR)
        return
    if not "x509PrivateKey" in account:
        printer.out("x509PrivateKey in ami account not found", printer.ERROR)
        return

    myCredAccount.accountNumber = account["accountNumber"]
    myCredAccount.name = account["name"]
    myCredAccount.accessKeyId = account["accessKeyId"]
    myCredAccount.secretAccessKeyId = account["secretAccessKeyId"]

    myCredAccount.certificates = pyxb.BIND()
    # A hack to avoid a toDOM, toXML bug
    myCredAccount.certificates._ExpandedName = pyxb.namespace.ExpandedName(Namespace, 'Certificates')

    try:
        cert = certificate()
        with open(account["x509Cert"], "r") as myfile:
            cert.content_ = myfile.read()
        cert.type = "x509"
        cert.type._ExpandedName = pyxb.namespace.ExpandedName(Namespace, 'string')
        cert.name = ntpath.basename(account["x509Cert"])
        myCredAccount.certificates.append(cert)

        cert = certificate()
        with open(account["x509PrivateKey"], "r") as myfile:
            cert.content_ = myfile.read()
        cert.type = "ec2PrivateKey"
        cert.type._ExpandedName = pyxb.namespace.ExpandedName(Namespace, 'string')
        cert.name = ntpath.basename(account["x509PrivateKey"])
        myCredAccount.certificates.append(cert)

    except IOError as e:
        printer.out("File error: " + str(e), printer.ERROR)
        return

    return myCredAccount


def azure(account):
    if not "publishsettings" in account:
        printer.out("Azure Resource Manager account")
        return azure_arm(account)
    else:
        printer.out("Azure classic account")
        return azure_classic(account)


def azure_arm(account):
    myCredAccount = CredAccountAzureResourceManager()
    # doing field verification
    if not "name" in account:
        printer.out("name for azure account not found", printer.ERROR)
        return
    if not "accountName" in account:
        printer.out("Storgae account name not found", printer.ERROR)
        return
    if not "accountKey" in account:
        printer.out("Storgae account key not found", printer.ERROR)
        return

    myCredAccount.name = account["name"]
    myCredAccount.accountName = account["accountName"]
    myCredAccount.accountKey = account["accountKey"]

    return myCredAccount


def azure_classic(account):
    myCredAccount = CredAccountAzure()
    # doing field verification
    if not "name" in account:
        printer.out("name for azure account not found", printer.ERROR)
        return
    if not "publishsettings" in account:
        printer.out("publishsettings in azure account not found", printer.ERROR)
        return

    myCredAccount.name = account["name"]
    myCredAccount.publishsettings = account["publishsettings"]

    myCredAccount.certificates = pyxb.BIND()
    # A hack to avoid a toDOM, toXML bug
    myCredAccount.certificates._ExpandedName = pyxb.namespace.ExpandedName(Namespace, 'Certificates')

    try:
        cert = certificate()
        with open(account["publishsettings"], "r") as myfile:
            cert.content_ = myfile.read()
        cert.type = "azurePublishSettings"
        cert.type._ExpandedName = pyxb.namespace.ExpandedName(Namespace, 'string')
        cert.name = ntpath.basename(account["publishsettings"])
        myCredAccount.certificates.append(cert)

    except IOError as e:
        printer.out("File error: " + str(e), printer.ERROR)
        return

    return myCredAccount


def eucalyptus(account):
    myCredAccount = CredAccountEws()
    # doing field verification
    if not "secretKey" in account:
        printer.out("secretKey in eucalyptus account not found", printer.ERROR)
        return
    if not "queryId" in account:
        printer.out("queryId in eucalyptus account not found", printer.ERROR)
        return
    if not "endpoint" in account:
        printer.out("endpoint in eucalyptus account not found", printer.ERROR)
        return
    if not "cloudCert" in account:
        printer.out("cloudCert in eucalyptus account not found", printer.ERROR)
        return
    if not "x509Cert" in account:
        printer.out("x509Cert in eucalyptus account not found", printer.ERROR)
        return
    if not "x509PrivateKey" in account:
        printer.out("x509PrivateKey in azure eucalyptus not found", printer.ERROR)
        return
    if not "accountNumber" in account:
        printer.out("accountNumber for eucalyptus account not found", printer.ERROR)
        return
    if not "name" in account:
        printer.out("name for eucalyptus account not found", printer.ERROR)
        return

    myCredAccount.accountNumber = account["accountNumber"]
    myCredAccount.name = account["name"]
    myCredAccount.hostname = account["endpoint"]
    myCredAccount.secretAccessKeyID = account["secretKey"]
    myCredAccount.accessKeyID = account["queryId"]

    myCertificates = certificates()
    myCredAccount.certificates = myCertificates

    try:
        myCertificate = certificate()
        with open(account["x509Cert"], "r") as myfile:
            myCertificate.certStr = myfile.read()
        myCertificate.type_ = "x509"
        myCertificate.name = ntpath.basename(account["x509Cert"])
        myCertificates.add_certificate(myCertificate)
        myCertificate = certificate()
        with open(account["x509PrivateKey"], "r") as myfile:
            myCertificate.certStr = myfile.read()
        myCertificate.type_ = "ec2PrivateKey"
        myCertificate.name = ntpath.basename(account["x509PrivateKey"])
        myCertificates.add_certificate(myCertificate)
        myCertificate = certificate()
        with open(account["cloudCert"], "r") as myfile:
            myCertificate.certStr = myfile.read()
        myCertificate.type_ = "eucCert"
        myCertificate.name = ntpath.basename(account["cloudCert"])
        myCertificates.add_certificate(myCertificate)


    except IOError as e:
        printer.out("File error: " + str(e), printer.ERROR)
        return

    return myCredAccount


def abiquo(account):
    myCredAccount = CredAccountAbiquo()
    # doing field verification
    if not "password" in account:
        printer.out("password in abiquo account not found", printer.ERROR)
        return
    if not "username" in account:
        printer.out("username in abiquo account not found", printer.ERROR)
        return
    if not "hostname" in account:
        printer.out("hostname for abiquo account not found", printer.ERROR)
        return
    if not "name" in account:
        printer.out("name for abiquo account not found", printer.ERROR)
        return

    myCredAccount.login = account["username"]
    myCredAccount.password = account["password"]
    myCredAccount.hostname = account["hostname"]
    myCredAccount.name = account["name"]
    return myCredAccount


def nimbula(account):
    myCredAccount = CredAccountNimbula()
    # doing field verification
    if not "password" in account:
        printer.out("password in nimbula account not found", printer.ERROR)
        return
    if not "username" in account:
        printer.out("username in nimbula account not found", printer.ERROR)
        return
    if not "endpoint" in account:
        printer.out("endpoint for nimbula account not found", printer.ERROR)
        return
    if not "name" in account:
        printer.out("name for nimbula account not found", printer.ERROR)
        return

    myCredAccount.login = account["username"]
    myCredAccount.password = account["password"]
    myCredAccount.serverUrl = account["endpoint"]
    myCredAccount.name = account["name"]
    return myCredAccount


def flexiant(account):
    myCredAccount = CredAccountFlexiant()
    # doing field verification
    if not "apiUsername" in account:
        printer.out("apiUsername in flexiant account not found", printer.ERROR)
        return
    if not "password" in account:
        printer.out("password in flexiant account not found", printer.ERROR)
        return
    if not "wsdlUrl" in account:
        printer.out("wsdlURL for flexiant account not found", printer.ERROR)
        return
    if not "name" in account:
        printer.out("name for flexiant account not found", printer.ERROR)
        return

    myCredAccount.apiUsername = account["apiUsername"]
    myCredAccount.password = account["password"]
    myCredAccount.wsdlUrl = account["wsdlUrl"]
    myCredAccount.name = account["name"]

    try:
        myCredAccount.userUUID = (myCredAccount.apiUsername).split('/')[1]
    except:
        printer.out(account["apiUsername"] + " is not a valid Flexiant username", printer.ERROR)
        return

    return myCredAccount


def vclouddirector(account):
    myCredAccount = CredAccountVCloudDirector()
    # doing field verification
    if not "name" in account:
        printer.out("name in vcd account not found", printer.ERROR)
        return
    if not "hostname" in account:
        printer.out("hostname in vcd account not found", printer.ERROR)
        return
    if not "login" in account:
        printer.out("login in vcd account not found", printer.ERROR)
        return
    if not "password" in account:
        printer.out("password in vcd account not found", printer.ERROR)
        return
    if not "organizationName" in account:
        printer.out("organizationName in vcd account not found", printer.ERROR)
        return

    if "port" in account:
        port = int(account["port"])
    else:
        port = 443

    myCredAccount.name = account["name"]
    myCredAccount.hostname = account["hostname"]
    myCredAccount.login = account["login"]
    myCredAccount.password = account["password"]
    myCredAccount.organizationName = account["organizationName"]
    myCredAccount.port = port

    return myCredAccount


def vsphere(account):
    myCredAccount = CredAccountVSphere()
    # doing field verification
    if not "name" in account:
        printer.out("name in vcenter account not found", printer.ERROR)
        return
    if not "login" in account:
        printer.out("login in vcenter account not found", printer.ERROR)
        return
    if not "password" in account:
        printer.out("password in vcenter account not found", printer.ERROR)
        return
    if not "hostname" in account:
        printer.out("hostname in vcenter account not found", printer.ERROR)
        return
    if "proxyHostname" in account:
        myCredAccount.proxyHost = account["proxyHostname"]
    if "proxyPort" in account:
        myCredAccount.proxyPort = account["proxyPort"]
    if "port" in account:
        port = int(account["port"])
    else:
        port = 443

    myCredAccount.name = account["name"]
    myCredAccount.login = account["login"]
    myCredAccount.password = account["password"]
    myCredAccount.hostname = account["hostname"]
    myCredAccount.port = port
    return myCredAccount


def gce(account):
    myCredAccount = CredAccountGoogle()
    # doing field verification
    if not "username" in account:
        printer.out("username in gce account not found", printer.ERROR)
        return
    if not "certPassword" in account:
        printer.out("certPassword in gce account not found", printer.ERROR)
        return
    if not "cert" in account:
        printer.out("cert in gce account not found", printer.ERROR)
        return
    if not "name" in account:
        printer.out("name for gce account not found", printer.ERROR)
        return

    myCredAccount.type_ = "google"
    myCredAccount.login = account["username"]
    myCredAccount.password = account["certPassword"]
    myCredAccount.name = account["name"]

    myCertificates = certificates()
    myCredAccount.certificates = myCertificates

    try:
        myCertificate = certificate()
        with open(account["cert"], "r") as myfile:
            myCertificate.certStr = myfile.read()
        myCertificate.type_ = "googleCertificate"
        myCertificate.name = ntpath.basename(account["cert"])
        myCertificates.add_certificate(myCertificate)

    except IOError as e:
        printer.out("File error: " + str(e), printer.ERROR)
        return

    return myCredAccount


def outscale(account):
    myCredAccount = CredAccountOutscale()
    # doing field verification
    if not "name" in account:
        printer.out("name for outscale account not found", printer.ERROR)
        return
    if not "accessKey" in account:
        printer.out("accessKey in outscale account not found", printer.ERROR)
        return
    if not "secretAccessKey" in account:
        printer.out("secretAccessKey in outscale account not found", printer.ERROR)
        return

    myCredAccount.secretAccessKeyID = account["secretAccessKey"]
    myCredAccount.accessKeyID = account["accessKey"]
    myCredAccount.name = account["name"]
    return myCredAccount

def k5(account):
    myCredAccount = CredAccountK5()
    # doing field verification
    if not "name" in account:
        printer.out("name for K5 account not found", printer.ERROR)
        return
    if not "login" in account:
        printer.out("login in K5 account not found", printer.ERROR)
        return
    if not "password" in account:
        printer.out("password in K5 account not found", printer.ERROR)
        return

    myCredAccount.name = account["name"]
    myCredAccount.login = account["login"]
    myCredAccount.password = account["password"]
    return myCredAccount

def docker(account):
    myCredAccount = CredAccountDocker()

    if not "name" in account:
        printer.out("name for Docker account is missing", printer.ERROR)
        return
    if not "endpointUrl" in account:
        printer.out("endpointUrl for Docker account is missing", printer.ERROR)
        return
    if not "login" in account:
        printer.out("login for Docker account is missing", printer.ERROR)
        return
    if not "password" in account:
        printer.out("password for Docker account is missing", printer.ERROR)
        return

    myCredAccount.name = account["name"]
    myCredAccount.endpointUrl = account["endpointUrl"]
    myCredAccount.login = account["login"]
    myCredAccount.password = account["password"]
    return myCredAccount


def get_target_platform_object(api, login, targetPlatformName):
    targetPlatformsUser = api.Users(login).Targetplatforms.Getall()
    if targetPlatformsUser is None or len(targetPlatformsUser.targetPlatforms.targetPlatform) == 0:
        return None
    else:
        for item in targetPlatformsUser.targetPlatforms.targetPlatform:
            if (item.name == targetPlatformName):
                return item
    return None


def assign_target_platform_account(credAccount, targetPlatformName):
    myTargetPlatform = targetPlatform()
    myTargetPlatform.name = targetPlatformName
    credAccount.targetPlatform = myTargetPlatform
    return credAccount



# -*- coding: utf-8 -*-
# Copyright 2007-2016 UShareSoft SAS, All rights reserved
#
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from unittest import TestCase

import pyxb
from hammr.commands.template import template
from uforge.application import Api
from uforge.objects.uforge import *
from mock import MagicMock
from mock import patch
from mock import ANY
from hammr.utils import constants
import datetime


class TestTemplate(TestCase):
    def test_do_clone_should_split_parameters_even_with_spaces(self):
        # given
        t = template.Template()
        name = "my new name wit spaces"
        args = "--id 42 --name '%s' --version 1.0" % name
        t.clone_appliance = MagicMock(return_value=appliance())

        # when
        t.do_clone(args)

        # then
        self.assertEqual(t.clone_appliance.call_count, 1)
        self.assertEqual(t.clone_appliance.call_args[0][1].name, name)

    @patch('uforge.application.Api._Users._Appliances.Getall')
    @patch('uforge.application.Api._Users._Images.Getall')
    @patch('texttable.Texttable.add_row')
    def test_do_list_succeed_when_appliance_list_with_UTC_Timezone(self, mock_table_add_row, mock_api_images_getall,mock_api_appliance_getall):
        # given
        t = template.Template()
        t.api = Api("url", username="username", password="password", headers=None, disable_ssl_certificate_validation = False, timeout = constants.HTTP_TIMEOUT)
        t.login = "login"
        t.password = "password"
        timeZone = "UTC"
        self.create_appliance_list(timeZone, mock_api_appliance_getall)
        new_images = Images()
        new_images.Images = pyxb.BIND()
        mock_api_images_getall.return_value = new_images

        # when
        t.do_list("")

        # then
        self.assertEquals(mock_table_add_row.call_count, 1)
        mock_table_add_row.assert_called_with([1, "ApplianceWithInstallProfileUTCTimezone", "1.0", "Ubuntu 14.x x86_64", ANY, ANY, 0, None, ANY, ANY])


    def create_appliance_list(self, timeZone, mock_api_appliance_getall):
            new_appliances = Appliances()
            new_appliances.appliances = pyxb.BIND()

            prof = InstallProfile()
            prof.timezone = timeZone

            newAppliance = Appliance()
            newAppliance.dbId = 1
            newAppliance.created = datetime.datetime.now()
            newAppliance.lastModified = datetime.datetime.now()
            newAppliance.name = "ApplianceWithInstallProfileUTCTimezone"
            newAppliance.version = "1.0"
            newAppliance.archName = "x86_64"
            newAppliance.distributionName = "Ubuntu 14.x"
            newAppliance.installProfile = prof

            new_appliances.appliances.append(newAppliance)

            mock_api_appliance_getall.return_value = new_appliances


# Copyright 2007-2015 UShareSoft SAS, All rights reserved
#
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from hurry.filesize import size

from ussclicore.argumentParser import ArgumentParser, ArgumentParserError
from ussclicore.cmd import Cmd, CoreGlobal
from ussclicore.utils import generics_utils, printer, ascii_bar_graph
from hammr.utils.hammr_utils import *
from hammr.utils import constants



class Quota(Cmd, CoreGlobal):
    """List the status of all the quotas that can be set for the user (disk usage, generations, scans and number of templates)"""

    cmd_name="quota"

    def __init__(self):
        super(Quota, self).__init__()

    def arg_list(self):
        doParser = ArgumentParser(prog=self.cmd_name+" list", add_help = True, description="Displays the user's quota information")
        return doParser

    def do_list(self, args):
        try:
            #call UForge API
            printer.out("Getting quotas for ["+self.login+"] ...")
            quotas = self.api.Users(self.login).Quotas.Get()
            if quotas is None or len(quotas.quotas.quota) == 0:
                printer.out("No quotas available")
            else:
                values = {}
                for quota in quotas.quotas.quota:
                    if quota.limit == -1:
                        nb = " (" + str(quota.nb) + ")"
                    else:
                        nb = " (" + str(quota.nb) + "/" + str(quota.limit) + ")"

                    if quota.type == constants.QUOTAS_SCAN:
                        text = "Scan" + ("s" if quota.nb > 1 else "") + nb
                    elif quota.type == constants.QUOTAS_TEMPLATE:
                        text = "Template" + ("s" if quota.nb > 1 else "") + nb
                    elif quota.type == constants.QUOTAS_GENERATION:
                        text = "Generation" + ("s" if quota.nb > 1 else "") + nb
                    elif quota.type == constants.QUOTAS_DISK_USAGE:
                        text = "Disk usage (" + size(quota.nb) + ")"

                    if quota.limit != -1:
                        nb = float(quota.nb)
                        limit = float(quota.limit)
                        values[text] = (nb/limit) * 50
                    else:
                        values[text] = -1

                ascii_bar_graph.print_graph(values)
            return 0

        except ArgumentParserError as e:
            printer.out("ERROR: In Arguments: "+str(e), printer.ERROR)
            self.help_list()
        except Exception as e:
            return handle_uforge_exception(e)

    def help_list(self):
        doParser = self.arg_list()
        doParser.print_help()



# Copyright 2007-2015 UShareSoft SAS, All rights reserved
#
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import urllib
import os.path
import shutil
import paramiko
import getpass
import shlex
import sys
import time

from ussclicore.argumentParser import ArgumentParser, ArgumentParserError
from ussclicore.cmd import Cmd, CoreGlobal
from progressbar import AnimatedMarker, Bar, BouncingBar, Counter, ETA, \
    FileTransferSpeed, FormatLabel, Percentage, \
    ProgressBar, ReverseBar, RotatingMarker, \
    SimpleProgress, Timer
from ussclicore.utils import generics_utils, printer, progressbar_widget, download_utils
from hammr.utils import *
from uforge.objects.uforge import *
from hammr.utils.hammr_utils import *


class Scan(Cmd, CoreGlobal):
    """List or delete existing scan images, build an image from a scan, launch the scan of a live system, or import a scan as an image"""

    cmd_name = "scan"

    def __init__(self):
        super(Scan, self).__init__()

    def arg_list(self):
        doParser = ArgumentParser(prog=self.cmd_name + " list", add_help=True,
                                  description="Displays all the scans for the user")
        return doParser

    def do_list(self, args):
        try:
            # call UForge API
            printer.out("Getting scans for [" + self.login + "] ...")
            myScannedInstances = self.api.Users(self.login).Scannedinstances.Getall(Includescans="true")
            myScannedInstances = myScannedInstances.scannedInstances.scannedInstance
            if myScannedInstances is None or len(myScannedInstances) == 0:
                printer.out("No scans available")
                return
            print scan_utils.scan_table(myScannedInstances).draw() + "\n"
            printer.out("Found " + str(len(myScannedInstances)) + " scans")
        except ArgumentParserError as e:
            printer.out("ERROR: In Arguments: " + str(e), printer.ERROR)
            self.help_list()
        except Exception as e:
            return handle_uforge_exception(e)

    def help_list(self):
        doParser = self.arg_list()
        doParser.print_help()

    def arg_run(self):
        doParser = ArgumentParser(prog=self.cmd_name + " run", add_help=True,
                                  description="Executes a deep scan of a running system")
        mandatory = doParser.add_argument_group("mandatory arguments")
        mandatory.add_argument('--ip', dest='ip', required=True,
                               help="the IP address or fully qualified hostname of the running system")
        mandatory.add_argument('--scan-login', dest='login', required=True, help="the root user name (normally root)")
        mandatory.add_argument('--name', dest='name', required=True,
                               help="the scan name to use when creating the scan meta-data")
        optional = doParser.add_argument_group("optional arguments")
        optional.add_argument('--scan-port', dest='port', required=False,
                              help="the ssh port of the running system")
        optional.add_argument('--scan-password', dest='password', required=False,
                              help="the root password to authenticate to the running system")
        optional.add_argument('--dir', dest='dir', required=False,
                              help="the directory where to install the uforge-scan.bin binary used to execute the deep scan")
        optional.add_argument('--exclude', dest='exclude', nargs='+', required=False,
                              help="a list of directories or files to exclude during the deep scan")
        optional.add_argument('-o', '--overlay', dest='overlay', action='store_true', required=False, help="perform a scan with overlay into the given scanned instance")
        return doParser

    def do_run(self, args):
        try:
            # add arguments
            doParser = self.arg_run()
            doArgs = doParser.parse_args(shlex.split(args))

            #if the help command is called, parse_args returns None object
            if not doArgs:
                    return 2

            if not self.check_overlay_option_is_allowed(doArgs.name, doArgs.overlay):
                return 2

            # download scan binary
            uri = generics_utils.get_uforge_url_from_ws_url(self.api.getUrl())
            download_url = uri + constants.URI_SCAN_BINARY

            if os.path.isdir(constants.TMP_WORKING_DIR):
                # delete tmp dir
                shutil.rmtree(constants.TMP_WORKING_DIR)
            os.mkdir(constants.TMP_WORKING_DIR)
            local_uforge_scan_path = constants.TMP_WORKING_DIR + os.sep + constants.SCAN_BINARY_NAME

            dlUtils = download_utils.Download(download_url, local_uforge_scan_path, not self.api.getDisableSslCertificateValidation())
            try:
                dlUtils.start()
            except Exception, e:
                return 2

            r_code = self.deploy_and_launch_agent(self.login, self.password, doArgs, local_uforge_scan_path,
                                                  self.api.getUrl())

            if r_code != 0:
                return

            # delete tmp dir
            shutil.rmtree(constants.TMP_WORKING_DIR)

            printer.out("Searching scan on uforge ...")
            running = True
            while running:
                myScannedInstances = self.api.Users(self.login).Scannedinstances.Getall(Includescans="true",
                                                                                    Name=doArgs.name)
                myScannedInstances = myScannedInstances.scannedInstances.scannedInstance
                if myScannedInstances is None or len(myScannedInstances) == 0:
                    time.sleep(5)
                else:
                    if len(myScannedInstances) > 1:
                        printer.out("A scan with the same name already exists", printer.ERROR)
                    myScannedInstance = myScannedInstances[0]
                    if len(myScannedInstance.scans.scan) == 0:
                        time.sleep(5)
                    else:
                        for scan in myScannedInstance.scans.scan:
                            if (not scan.status.complete and not scan.status.error and not scan.status.cancelled):
                                status = scan.status
                                statusWidget = progressbar_widget.Status()
                                statusWidget.status = status
                                widgets = [Bar('>'), ' ', statusWidget, ' ', ReverseBar('<')]
                                progress = ProgressBar(widgets=widgets, maxval=100).start()
                                while not (status.complete or status.error or status.cancelled):
                                    statusWidget.status = status
                                    progress.update(status.percentage)
                                    status = (self.api.Users(self.login).Scannedinstances(myScannedInstance.dbId).Scans(
                                        scan.dbId).Get("false", "false", "false", "false", None, None, None, None,
                                                       None)).status
                                    time.sleep(2)
                                statusWidget.status = status
                                progress.finish()
                                if status.error:
                                    printer.out("Scan  error: " + status.message + "\n" + status.errorMessage,
                                                printer.ERROR)
                                    if status.detailedError:
                                        printer.out(status.detailedErrorMsg)
                                elif status.cancelled:
                                    printer.out("Scan canceled: " + status.message, printer.WARNING)
                                else:
                                    printer.out("Scan successfully", printer.OK)
                                running = False
                                break
                            else:
                                pass


        except ArgumentParserError as e:
            printer.out("ERROR: In Arguments: " + str(e), printer.ERROR)
            self.help_run()
        except Exception as e:
            return handle_uforge_exception(e)

    def help_run(self):
        doParser = self.arg_run()
        doParser.print_help()

    def check_overlay_option_is_allowed(self, name, overlay):
        myScannedInstance = self.api.Users(self.login).Scannedinstances.Getall(Includescans="true", Name=name)
        myScannedInstance = myScannedInstance.scannedInstances.scannedInstance
        if myScannedInstance is not None and len(myScannedInstance) > 0:
            myScannedInstance = myScannedInstance[0]
            if ((overlay and not myScannedInstance.overlayIncluded) or (
                not overlay and myScannedInstance.overlayIncluded)):
                scanTypeString = 'regular scan'
                if overlay:
                    scanTypeString = 'scan with overlay'
                printer.out(
                    "Performing {0} into the scanned instance [{1}] is not allowed. Please retry with another one.".format(
                        scanTypeString, name), printer.ERROR)
                return False
        return True

    def arg_build(self):
        doParser = ArgumentParser(prog=self.cmd_name + " build", add_help=True,
                                  description="Builds a machine image from a scan")
        mandatory = doParser.add_argument_group("mandatory arguments")
        mandatory.add_argument('--id', dest='id', required=True,
                               help="the ID of the scan to generate the machine image from")
        mandatory.add_argument('--file', dest='file', required=True, help="yaml/json file providing the builder parameters")
        return doParser

    def do_build(self, args):
        try:
            # add arguments
            doParser = self.arg_build()
            try:
                doArgs = doParser.parse_args(shlex.split(args))
            except SystemExit as e:
                return
            # --
            file = generics_utils.get_file(doArgs.file)
            if file is None:
                return 2
            data = load_data(file)
            if "builders" in data:
                builders = hammr_utils.check_mandatory_builders(data["builders"])
                builders = hammr_utils.check_mandatory_generate_scan(builders)
            else:
                printer.out("no builder section found", printer.ERROR)
                return
            if builders is None:
                return
            try:
                myScannedInstances = self.api.Users(self.login).Scannedinstances.Getall(Includescans="true")
                if myScannedInstances is None or not hasattr(myScannedInstances, 'scannedInstances'):
                    printer.out("scan not found", printer.ERROR)
                    return
                else:
                    myScan = None
                    for myScannedInstance in myScannedInstances.scannedInstances.scannedInstance:
                        for scan in myScannedInstance.scans.scan:
                            if str(scan.dbId) == doArgs.id:
                                myScan = scan
                                myRScannedInstance = myScannedInstance
                                break
                        if myScan is not None:
                            break

                if myScan is not None and myScan.status.complete and not myScan.status.error and not myScan.status.cancelled:
                    i = 1
                    for builder in builders:
                        printer.out(
                            "Generating '" + builder["type"] + "' image (" + str(i) + "/" + str(len(builders)) + ")")
                        format_type = builder["type"]
                        targetFormat = generate_utils.get_target_format_object(self.api, self.login, format_type)

                        if targetFormat is None:
                            printer.out("Builder type unknown: "+format_type, printer.ERROR)
                            return 2
                        myimage = image()

                        myinstallProfile = installProfile()
                        if "swapSize" in builder["installation"]:
                            myinstallProfile.swapSize = builder["installation"]["swapSize"]
                        myinstallProfile.diskSize = builder["installation"]["diskSize"]


                        func = getattr(generate_utils, "generate_"+generics_utils.remove_special_chars(targetFormat.format.name), None)
                        if func:
                            myimage, myinstallProfile = func(myimage, builder, myinstallProfile, self.api, self.login)
                        else:
                            printer.out("Builder type unknown: "+format_type, printer.ERROR)
                            return 2

                        myimage.targetFormat = targetFormat
                        myimage.installProfile = myinstallProfile
                        rImage = self.api.Users(self.login).Scannedinstances(myRScannedInstance.dbId).Scans(
                            myScan.dbId).Images().Generate(myimage)
                        status = rImage.status
                        statusWidget = progressbar_widget.Status()
                        statusWidget.status = status
                        widgets = [Bar('>'), ' ', statusWidget, ' ', ReverseBar('<')]
                        progress = ProgressBar(widgets=widgets, maxval=100).start()
                        while not (status.complete or status.error or status.cancelled):
                            statusWidget.status = status
                            progress.update(status.percentage)
                            status = self.api.Users(self.login).Scannedinstances(myRScannedInstance.dbId).Scans(
                                myScan.dbId).Images(Sitid=rImage.dbId).Status.Get()
                            time.sleep(2)
                        statusWidget.status = status
                        progress.finish()
                        if status.error:
                            printer.out("Generation '" + builder[
                                "type"] + "' error: " + status.message + "\n" + status.errorMessage, printer.ERROR)
                            if status.detailedError:
                                printer.out(status.detailedErrorMsg)
                        elif status.cancelled:
                            printer.out("Generation '" + builder["type"] + "' canceled: " + status.message,
                                        printer.ERROR)
                        else:
                            printer.out("Generation '" + builder["type"] + "' ok", printer.OK)
                        i += 1
                else:
                    printer.out("Impossible to generate this scan", printer.ERROR)

            except KeyError as e:
                printer.out("unknown error template file", printer.ERROR)




        except ArgumentParserError as e:
            printer.out("ERROR: In Arguments: " + str(e), printer.ERROR)
            self.help_build()
        except KeyboardInterrupt:
            printer.out("\n")
            if generics_utils.query_yes_no("Do you want to cancel the job ?"):
                if 'myRScannedInstance' in locals() and 'myScan' in locals() and 'rImage' in locals() \
                        and hasattr(myRScannedInstance, 'dbId') and hasattr(myScan, 'dbId') and hasattr(rImage, 'dbId'):
                    self.api.Users(self.login).Scannedinstances(myRScannedInstance.dbId).Scans(myScan.dbId).Images(
                        Sitid=rImage.dbId).Status.Cancel()
            else:
                print "Exiting command"
        except Exception as e:
            return handle_uforge_exception(e)

    def help_build(self):
        doParser = self.arg_build()
        doParser.print_help()

    def arg_import(self):
        doParser = ArgumentParser(prog=self.cmd_name + " import", add_help=True,
                                  description="Imports (or transforms) the scan to a template")
        mandatory = doParser.add_argument_group("mandatory arguments")
        mandatory.add_argument('--id', dest='id', required=True, help="the ID of the scan to import")
        mandatory.add_argument('--name', dest='name', required=True, nargs='+',
                               help="the name to use for the template created from the scan")
        mandatory.add_argument('--version', dest='version', required=True,
                               help="the version to use for the template created from the scan")
        return doParser

    def do_import(self, args):
        try:
            # add arguments
            doParser = self.arg_import()
            try:
                doArgs = doParser.parse_args(shlex.split(args))
                doArgs.name = " ".join(doArgs.name)
            except SystemExit as e:
                return

            printer.out("Import scan id [" + doArgs.id + "] ...")
            myScannedInstances = self.api.Users(self.login).Scannedinstances.Getall(Includescans="true")
            if myScannedInstances is None or not hasattr(myScannedInstances, 'scannedInstances'):
                printer.out("scan not found", printer.ERROR)
                return
            else:
                myScan = None
                for myScannedInstance in myScannedInstances.scannedInstances.scannedInstance:
                    for scan in myScannedInstance.scans.scan:
                        if str(scan.dbId) == doArgs.id:
                            myScan = scan
                            myRScannedInstance = myScannedInstance
                            break
                    if myScan is not None:
                        break

            if myScan is not None and myScan.status.complete and not myScan.status.error and not myScan.status.cancelled:
                myScanImport = scanImport()
                myScanImport.importedObjectName = doArgs.name
                myScanImport.importedObjectVersion = doArgs.version
                myScanImport.orgUri = (self.api.Users(self.login).Orgs().Getall()).orgs.org[0].uri
                rScanImport = self.api.Users(self.login).Scannedinstances(myRScannedInstance.dbId).Scans(
                    myScan.dbId).Imports().Import(myScanImport)
                status = rScanImport.status
                statusWidget = progressbar_widget.Status()
                statusWidget.status = status
                widgets = [Bar('>'), ' ', statusWidget, ' ', ReverseBar('<')]
                progress = ProgressBar(widgets=widgets, maxval=100).start()
                while not (status.complete or status.error or status.cancelled):
                    statusWidget.status = status
                    progress.update(status.percentage)
                    status = (self.api.Users(self.login).Scannedinstances(myRScannedInstance.dbId).Scans(
                        myScan.dbId).Imports().Status.Get(I=rScanImport.uri)).statuses.status[0]
                    time.sleep(2)
                statusWidget.status = status
                progress.finish()
                if status.error:
                    printer.out("Importing error: " + status.message + "\n" + status.errorMessage, printer.ERROR)
                    if status.detailedError:
                        printer.out(status.detailedErrorMsg)
                elif status.cancelled:
                    printer.out("Importing canceled: " + status.message, printer.WARNING)
                else:
                    printer.out("Importing ok", printer.OK)

        except KeyboardInterrupt:
            printer.out("\n")
            if generics_utils.query_yes_no("Do you want to cancel the job ?"):
                if 'myRScannedInstance' in locals() and 'myScan' in locals() and 'rScanImport' in locals() \
                        and hasattr(myRScannedInstance, 'dbId') and hasattr(myScan, 'dbId') and hasattr(rScanImport,
                                                                                                        'dbId'):
                    self.api.Users(self.login).Scannedinstances(myRScannedInstance.dbId).Scans(myScan.dbId).Imports(
                        rScanImport.dbId).Status.Cancel()
            else:
                printer.out("Exiting command")
        except ArgumentParserError as e:
            printer.out("ERROR: In Arguments: " + str(e), printer.ERROR)
            self.help_import()
        except Exception as e:
            return handle_uforge_exception(e)

    def help_import(self):
        doParser = self.arg_import()
        doParser.print_help()

    def arg_delete(self):
        doParser = ArgumentParser(prog=self.cmd_name + " delete", add_help=True, description="Deletes an existing instance or scan")
        mandatory = doParser.add_argument_group("mandatory arguments")
        mandatory.add_argument('--id', dest='id', required=True, help="the ID of the instance or scan to delete")
        optional = doParser.add_argument_group("optional arguments")
        optional.add_argument('--scantype', dest='scantype', required=False,
                              help="the scan type, values is [instance|scan|all] (default is scan)")
        optional.add_argument('--scansonly', dest='scansonly', required=False, action='store_true',
                              help="if scan type is instance, and this argument is used, will only remove the scans not the instance itself")
        return doParser

    def do_delete(self, args):
        try:
            doParser = self.arg_delete()
            try:
                doArgs = doParser.parse_args(shlex.split(args))
            except SystemExit as e:
                return
            # call UForge API
            searchedScanType = 'scan'
            extraInfo = "Retrieving scan with id [" + doArgs.id + "] ..."
            if doArgs.scantype:
                if doArgs.scantype != 'scan' and doArgs.scantype != 'instance' and doArgs.scantype != 'all':
                    printer.out(
                        "ERROR: scantype can only be 'scan', 'instance' or 'all' not: '" + doArgs.scantype + "'",
                        printer.ERROR)
                    return
                searchedScanType = doArgs.scantype
            if searchedScanType != 'instance' and doArgs.scansonly:
                printer.out(
                    "ERROR: 'scansonly' can only be used with 'instance' scantype but not with '" + searchedScanType + "'",
                    printer.ERROR)
                return
            if searchedScanType == 'instance':
                extraInfo = "Retrieving scan instance with id [" + doArgs.id + "] ..."
            else:
                if searchedScanType == 'all':
                    extraInfo = 'Retrieving all scan instances and associated scans'
            printer.out(extraInfo)
            myScannedInstances = self.api.Users(self.login).Scannedinstances.Getall(Includescans="true")
            myScannedInstances = myScannedInstances.scannedInstances.scannedInstance
            if myScannedInstances is None or len(myScannedInstances) == 0:
                printer.out("Nothing found")
                return
            if searchedScanType == 'all':
                print scan_utils.scan_table(myScannedInstances).draw() + "\n"
                if generics_utils.query_yes_no("Do you really want to delete all scan instances"):
                    printer.out("Please wait...")
                    self.api.Users(self.login).Scannedinstances().Deleteall()
                    printer.out("All instances and scans deleted", printer.OK)
                return
            for myScannedInstance in myScannedInstances:
                if searchedScanType == 'instance' and str(myScannedInstance.dbId) == doArgs.id:
                    print scan_utils.scan_table([myScannedInstance]).draw() + "\n"
                    if doArgs.scansonly:
                        extraInfo = "Do you really want to delete all scans in instance with id " + str(doArgs.id)
                    else:
                        extraInfo = "Do you really want to delete scan instance with id " + str(
                            doArgs.id) + " and all associated scans"
                    if generics_utils.query_yes_no(extraInfo):
                        printer.out("Please wait...")
                        if doArgs.scansonly:
                            self.api.Users(self.login).Scannedinstances(doArgs.id).Scans().Deleteall()
                            printer.out("Instance scans deleted", printer.OK)
                        else:
                            self.api.Users(self.login).Scannedinstances(doArgs.id).Delete()
                            printer.out("Instance deleted", printer.OK)
                        return
                    return
                if searchedScanType == 'scan':
                    for scan in myScannedInstance.scans.scan:
                        if str(scan.dbId) == doArgs.id:
                            print scan_utils.scan_table([myScannedInstance], scan).draw() + "\n"
                            if generics_utils.query_yes_no(
                                            "Do you really want to delete scan with id " + str(doArgs.id)):
                                printer.out("Please wait...")
                                self.api.Users(self.login).Scannedinstances(myScannedInstance.dbId).Scans(
                                    doArgs.id).Delete()
                                printer.out("Scan deleted", printer.OK)
                            return
            printer.out("Scan not found", printer.ERROR)
        except ArgumentParserError as e:
            printer.out("ERROR: In Arguments: " + str(e), printer.ERROR)
            self.help_delete()
        except Exception as e:
            return handle_uforge_exception(e)

    def help_delete(self):
        doParser = self.arg_delete()
        doParser.print_help()

    def deploy_and_launch_agent(self, uforge_login, uforge_password, args, file_src_path, uforge_url):
        hostname = args.ip
        username = args.login
        if not args.password:
            passW = getpass.getpass('Password for %s@%s: ' % (username, hostname))
        else:
            passW = args.password
        if not args.port:
            port = 22
        else:
            port = int(args.port)

        # paramiko.util.log_to_file('/tmp/ssh.log') # sets up logging

        # get host key, if we know one
        hostkeytype = None
        hostkey = None
        try:
            host_keys = paramiko.util.load_host_keys(os.path.expanduser('~/.ssh/known_hosts'))
        except IOError:
            try:
                # try ~/ssh/ too, because windows can't have a folder named ~/.ssh/
                host_keys = paramiko.util.load_host_keys(os.path.expanduser('~/ssh/known_hosts'))
            except IOError:
                printer.out("Unable to open host keys file", printer.ERROR)
                host_keys = {}


        # if host_keys.has_key(hostname):
        #        hostkeytype = host_keys[hostname].keys()[0]
        #        hostkey = host_keys[hostname][hostkeytype]
        #        print 'Using host key of type %s' % hostkeytype

        # now, connect and use paramiko Transport to negotiate SSH2 across the connection
        try:
            if not args.dir:
                dir = "/tmp"
            else:
                dir = args.dir
            t = paramiko.Transport((hostname, port))
            t.connect(username=username, password=passW, hostkey=hostkey)
            sftp = paramiko.SFTPClient.from_transport(t)

            # upload uforge scan binary
            sftp.put(file_src_path, dir + "/" + constants.SCAN_BINARY_NAME)
            t.close()

            client = paramiko.SSHClient()
            client.load_system_host_keys()
            client.set_missing_host_key_policy(paramiko.MissingHostKeyPolicy())
            client.connect(hostname, port, username, passW)

            # test service
            stdin, stdout, stderr = client.exec_command(
                'chmod +x ' + dir + '/' + constants.SCAN_BINARY_NAME + '; ' + dir + '/' + constants.SCAN_BINARY_NAME + ' -u ' + uforge_login + ' -p ' + uforge_password + ' -U ' + uforge_url + ' -P')
            for line in stdout:
                print '... ' + line.strip('\n')
            # launch scan
            exclude = ""
            if args.exclude:
                for ex in args.exclude:
                    exclude += "-e " + ex + " "
            overlay = ""
            if args.overlay:
                overlay = "-o"
            client.exec_command(
                'chmod +x ' + dir + '/' + constants.SCAN_BINARY_NAME + '; nohup ' + dir + '/' + constants.SCAN_BINARY_NAME + ' -u ' + uforge_login + ' -p ' + uforge_password + ' -U ' + uforge_url + ' ' + overlay + ' -n \'' + args.name + '\' ' + exclude + ' >/dev/null 2>&1 &')
            client.close()

        except paramiko.AuthenticationException as e:
            printer.out("Authentification error: " + e[0], printer.ERROR)
            return 2
        except Exception, e:
            printer.out("Caught exception: " + str(e), printer.ERROR)
            # traceback.print_exc()
            try:
                t.close()
                client.close()
            except:
                pass
            return 2

        return 0


# -*- coding: utf-8 -*-
# Copyright 2007-2016 UShareSoft SAS, All rights reserved
#
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from unittest import TestCase

import pyxb
from mock import patch
from mock import ANY
from uforge.application import Api
from uforge.objects import uforge

from hammr.commands.image import image

from hurry.filesize import size
from hammr.utils import constants
import datetime

class TestImage(TestCase):

    @patch('uforge.application.Api._Users._Images.Getall')
    @patch('uforge.application.Api._Users._Pimages.Getall')
    @patch('texttable.Texttable.add_row')
    def test_do_list_check_size(self, mock_table_add_row, mock_api_pimg_getall, mock_api_getall):
        # given
        i = image.Image()
        i.api = Api("url", username="username", password="password", headers=None,
                    disable_ssl_certificate_validation=False, timeout=constants.HTTP_TIMEOUT)
        i.login = "login"
        i.password = "password"
        self.create_image(6000, mock_api_getall)
        new_pimages = uforge.publishImages()
        new_pimages.publishImages = pyxb.BIND()
        mock_api_pimg_getall.return_value = new_pimages
        # when
        i.do_list("")

        # then
        self.assertEquals(mock_table_add_row.call_count, 1)
        mock_table_add_row.assert_called_with([ANY, ANY, ANY, ANY, ANY, ANY, size(6000), ANY, ANY])




    def create_image(self, size, mock_api_getall):
        new_images = uforge.images()
        new_images.images = pyxb.BIND()

        newImage = uforge.image()
        newImage.dbId = 1
        newImage.fileSize = size
        newImage.size = 0
        newImage.name = "test"
        newImage.status = "complete"
        newImage.targetFormat = uforge.targetFormat()
        newImage.targetFormat.name = "test"
        newImage.created = datetime.datetime.now()
        newImage.compress = True

        new_images.images.append(newImage)

        mock_api_getall.return_value = new_images



# Copyright 2007-2015 UShareSoft SAS, All rights reserved
#
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import shlex
from texttable import Texttable
from ussclicore.argumentParser import ArgumentParser, ArgumentParserError
from ussclicore.cmd import Cmd, CoreGlobal
from ussclicore.utils import generics_utils, printer
from hammr.utils.hammr_utils import *
from uforge.objects.uforge import *
from hammr.utils import account_utils


class Account(Cmd, CoreGlobal):
    """List, create or delete a cloud account"""

    cmd_name = "account"

    def __init__(self):
        super(Account, self).__init__()

    def arg_list(self):
        doParser = ArgumentParser(prog=self.cmd_name + " list", add_help=True,
                                  description="Displays all the cloud accounts for the user")
        return doParser

    def do_list(self, args):
        try:
            # call UForge API
            printer.out("Getting all your accounts ...")
            accounts = self.api.Users(self.login).Accounts.Getall()
            accounts = accounts.credAccounts.credAccount
            if accounts is None or len(accounts) == 0:
                printer.out("No accounts available")
            else:
                table = Texttable(800)
                table.set_cols_dtype(["t", "t", "t", "t"])
                table.header(["Id", "Name", "Type", "Created"])
                accounts = generics_utils.order_list_object_by(accounts, "name")
                for account in accounts:
                    table.add_row(
                        [account.dbId, account.name, account.targetPlatform.name, account.created.strftime("%Y-%m-%d %H:%M:%S")])
                print table.draw() + "\n"
                printer.out("Found " + str(len(accounts)) + " accounts")
            return 0
        except Exception as e:
            return handle_uforge_exception(e)

    def help_list(self):
        doParser = self.arg_list()
        doParser.print_help()

    def arg_create(self):
        doParser = ArgumentParser(prog=self.cmd_name + " create", add_help=True,
                                  description="Creates a new cloud account")
        mandatory = doParser.add_argument_group("mandatory arguments")
        mandatory.add_argument('--file', dest='file', required=True,
                               help="yaml/json file providing the cloud account parameters")
        return doParser

    def do_create(self, args):
        try:
            # add arguments
            doParser = self.arg_create()
            doArgs = doParser.parse_args(shlex.split(args))

            #if the help command is called, parse_args returns None object
            if not doArgs:
                    return 2

            # --
            file = generics_utils.get_file(doArgs.file)
            if file is None:
                return 2
            data = load_data(file)
            if data is None:
                return 2
            if "builders" in data:
                accounts_file_type = "builders"
                iterables = check_mandatory_create_account(data["builders"], accounts_file_type)
            elif "accounts" in data:
                accounts_file_type = "accounts"
                iterables = check_mandatory_create_account(data["accounts"], accounts_file_type)
            else:
                printer.out("Error: no builders or accounts section found", printer.ERROR)
                return 2
            if iterables is None:
                return 2
            try:
                for iterable in iterables:
                    myCredAccount = None
                    if "account" in iterable:
                        account_type = iterable["account"]["type"]
                    elif "type" in iterable:
                        account_type = iterable["type"]

                    targetPlatform = account_utils.get_target_platform_object(self.api, self.login, account_type)
                    if targetPlatform is None:
                        printer.out("Platform type unknown: "+account_type, printer.ERROR)
                        return 2

                    func = getattr(account_utils, generics_utils.remove_special_chars(targetPlatform.type), None)
                    if func:
                        if accounts_file_type == "builders" and "account" in iterable:
                            myCredAccount = func(iterable["account"])
                        elif accounts_file_type == "accounts":
                            myCredAccount = func(iterable)
                        else:
                            pass
                            # DO NOTHING - no account in builder to create

                            # TODO
                            # the account type must be in the account part, if no builder part (independant file)
                        if myCredAccount is not None:
                            myCredAccount = account_utils.assign_target_platform_account(myCredAccount, targetPlatform.name)
                            printer.out("Create account for '" + account_type + "'...")
                            self.api.Users(self.login).Accounts.Create(body=myCredAccount, element_name="ns1:credAccount")
                            printer.out("Account create successfully for [" + account_type + "]", printer.OK)
                return 0

            except KeyError as e:
                printer.out("unknown error template file", printer.ERROR)

        except IOError as e:
            printer.out("File error: " + str(e), printer.ERROR)
        except ArgumentParserError as e:
            printer.out("ERROR: In Arguments: " + str(e), printer.ERROR)
            self.help_create()
        except Exception as e:
            return handle_uforge_exception(e)

    def help_create(self):
        doParser = self.arg_create()
        doParser.print_help()

    def arg_delete(self):
        doParser = ArgumentParser(prog=self.cmd_name + " delete", add_help=True,
                                  description="Deletes an existing cloud account")
        mandatory = doParser.add_argument_group("mandatory arguments")
        mandatory.add_argument('--id', dest='id', required=True, help="the ID of the cloud account to delete")
        optional = doParser.add_argument_group("optional arguments")
        optional.add_argument('--no-confirm', dest='no_confirm', action='store_true', required=False,
                              help="do not ask before delete the cloud account")
        doParser.set_defaults(no_confirm=False)
        return doParser

    def do_delete(self, args):
        try:
            # add arguments
            doParser = self.arg_delete()
            doArgs = doParser.parse_args(shlex.split(args))

            #if the help command is called, parse_args returns None object
            if not doArgs:
                    return 2

            # call UForge API
            printer.out("Searching account with id [" + doArgs.id + "] ...")
            account = self.api.Users(self.login).Accounts(doArgs.id).Get()
            if account is None:
                printer.out("No Account available", printer.WARNING)
            else:
                table = Texttable(800)
                table.set_cols_dtype(["t", "t", "t", "t"])
                table.header(["Id", "Name", "Type", "Created"])
                table.add_row(
                    [account.dbId, account.name, account.targetPlatform.name, account.created.strftime("%Y-%m-%d %H:%M:%S")])
                print table.draw() + "\n"
                if doArgs.no_confirm:
                    self.api.Users(self.login).Accounts(doArgs.id).Delete()
                    printer.out("Account deleted", printer.OK)
                elif generics_utils.query_yes_no("Do you really want to delete account with id " + str(account.dbId)):
                    self.api.Users(self.login).Accounts(doArgs.id).Delete()
                    printer.out("Account deleted", printer.OK)
            return 0

        except ArgumentParserError as e:
            printer.out("ERROR: In Arguments: " + str(e), printer.ERROR)
            self.help_delete()
        except Exception as e:
            return handle_uforge_exception(e)

    def help_delete(self):
        doParser = self.arg_delete()
        doParser.print_help()



__author__ = 'UshareSoft'

import unittest
import json
import yaml
from mock import patch

from ussclicore.utils import generics_utils
from hammr.utils import bundle_utils

class TestFiles(unittest.TestCase):

    @patch("ussclicore.utils.printer.out")
    def test_check_bundle_should_failed_when_no_name(self, mock_method):
        #Given
        jsonPath = "tests/integration/data/bundle/bundleWithoutName.json"
        #When
        bundle = generics_utils.check_json_syntax(jsonPath)
        bundle_utils.check_bundle(bundle)
        #Then
        mock_method.assert_called_with("There is no attribute [name] for a [bundle]", "ERROR")

    @patch("ussclicore.utils.printer.out")
    def test_check_bundle_should_failed_when_no_version(self, mock_method):
        #Given
        jsonPath = "tests/integration/data/bundle/bundleWithoutVersion.json"
        #When
        bundle = generics_utils.check_json_syntax(jsonPath)
        bundle_utils.check_bundle(bundle)
        #Then
        mock_method.assert_called_with("no attribute [version] for [bundle]", "ERROR")

    @patch("ussclicore.utils.printer.out")
    def test_check_bundle_should_failed_when_no_files(self, mock_method):
        #Given
        jsonPath = "tests/integration/data/bundle/bundleWithoutFiles.json"
        #When
        bundle = generics_utils.check_json_syntax(jsonPath)
        bundle_utils.check_bundle(bundle)
        #Then
        mock_method.assert_called_with("no attribute [files] for [bundle]", "ERROR")

    @patch("ussclicore.utils.printer.out")
    def test_check_bundle_should_failed_when_files_no_name(self, mock_method):
        #Given
        jsonPath = "tests/integration/data/bundle/bundleFilesWithoutName.json"
        #When
        bundle = generics_utils.check_json_syntax(jsonPath)
        bundle_utils.check_bundle(bundle)
        #Then
        mock_method.assert_called_with("There is no attribute [name] for a [file]", "ERROR")

    @patch("ussclicore.utils.printer.out")
    def test_check_bundle_should_failed_when_files_no_source(self, mock_method):
        #Given
        jsonPath = "tests/integration/data/bundle/bundleFilesWithoutSource.json"
        #When
        bundle = generics_utils.check_json_syntax(jsonPath)
        bundle_utils.check_bundle(bundle)
        #Then
        mock_method.assert_called_with("There is no attribute [source] for a [file]", "ERROR")

    @patch("ussclicore.utils.printer.out")
    def test_check_bundle_should_failed_when_tag_softwarefile_and_bootorder(self, mock_method):
        #Given
        jsonPath = "tests/integration/data/bundle/bundleFilesTagSoftwareFileKeyBootOrder.json"
        #When
        bundle = generics_utils.check_json_syntax(jsonPath)
        bundle_utils.check_bundle(bundle)
        #Then
        mock_method.assert_called_with("There is the attribute [bootOrder] or [bootType] for file 'directoryTest' but is not tagged as 'bootscript'", "ERROR")

    @patch("ussclicore.utils.printer.out")
    def test_check_bundle_should_failed_when_tag_bootscript_and_rights(self, mock_method):
        #Given
        jsonPath = "tests/integration/data/bundle/bundleFilesTagBootScriptKeyRights.json"
        #When
        bundle = generics_utils.check_json_syntax(jsonPath)
        bundle_utils.check_bundle(bundle)
        #Then
        mock_method.assert_called_with("There is the attribute [ownerGroup], [rights] or [symlink] for file 'cleanup_tmp.sh' but is not tagged as 'softwarefile'", "ERROR")

    @patch("ussclicore.utils.printer.out")
    def test_check_bundle_should_failed_when_tag_ospkg_in_directory(self, mock_method):
        #Given
        jsonPath = "tests/integration/data/bundle/bundleFilesTagOSPkgInDirectory.json"
        #When
        bundle = generics_utils.check_json_syntax(jsonPath)
        bundle_utils.check_bundle(bundle)
        #Then
        mock_method.assert_called_with("The file 'iotop-0.6-2.el7.noarch.rpm, with tag 'ospkg' must be in the first level files section", "ERROR")

    def test_check_bundle_should_succeed_when_no_restrictionRule(self):
        # Given
        jsonPath = "tests/integration/data/bundle/bundleWithoutRestrictionRule.json"
        # When
        bundle = generics_utils.check_json_syntax(jsonPath)
        bundle_utils.check_bundle(bundle)
        # Then
        self.assertIsNotNone(bundle)

    def test_check_bundle_should_succeed_when_empty_restrictionRule(self):
        # Given
        jsonPath = "tests/integration/data/bundle/bundleWithEmptyRestrictionRule.json"
        # When
        bundle = generics_utils.check_json_syntax(jsonPath)
        bundle_utils.check_bundle(bundle)
        # Then
        self.assertIsNotNone(bundle)

    def test_check_bundle_should_succeed(self):
        #Given
        jsonPath = "tests/integration/data/bundle/bundleFull.json"
        #When
        bundle = generics_utils.check_json_syntax(jsonPath)
        bundle = bundle_utils.check_bundle(bundle)
        #Then
        self.assertIsNotNone(bundle)

if __name__ == '__main__':
    unittest.main()





# Copyright 2007-2015 UShareSoft SAS, All rights reserved
#
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from texttable import Texttable
from ussclicore.argumentParser import ArgumentParser, ArgumentParserError
from ussclicore.cmd import Cmd, CoreGlobal
from ussclicore.utils import generics_utils, printer
from hammr.utils.hammr_utils import *

class User(Cmd, CoreGlobal):
    """Lists information about a user, including login, email, name, date created, and status"""

    cmd_name="user"

    def __init__(self):
        super(User, self).__init__()

    def arg_info(self):
        doParser = ArgumentParser(prog=self.cmd_name+" info", add_help = True, description="Displays informations of provided user")
        return doParser

    def do_info(self, args):
        try:
            #call UForge API
            printer.out("Getting user ["+self.login+"] ...")
            user = self.api.Users(self.login).Get()
            if user is None:
                printer.out("user "+ self.login +"does not exist", printer.ERROR)
            else:
                table = Texttable(200)
                table.set_cols_align(["c", "l", "c", "c", "c", "c", "c", "c"])
                table.header(["Login", "Email", "Lastname",  "Firstname",  "Created", "Active", "Promo Code", "Creation Code"])
                table.add_row([user.loginName, user.email, user.surname , user.firstName, user.created.strftime("%Y-%m-%d %H:%M:%S"), "X", user.promoCode, user.creationCode])
                print table.draw() + "\n"
            return 0
        except ArgumentParserError as e:
            printer.out("ERROR: In Arguments: "+str(e), printer.ERROR)
            self.help_info()
        except Exception as e:
            return handle_uforge_exception(e)

    def help_info(self):
        doParser = self.arg_info()
        doParser.print_help()
                


# Copyright 2007-2015 UShareSoft SAS, All rights reserved
#
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from setuptools import setup,find_packages,Command
from hammr.utils.constants import *
import os


ROOT_DIR = os.path.dirname(os.path.realpath(__file__))


# Declare your packages' dependencies here, for eg:
# Always put an '==' dependency with uforge_python_sdk
requires=['uforge_python_sdk==3.7.6-RC3',
                    'httplib2==0.9',
                    'texttable>=0.8.1',
                    'progressbar==2.3',
                    'argparse',
                    'paramiko==1.12',
                    'pyparsing==2.0.2',
                    'pyyaml==3.12',
                    'hurry.filesize==0.9',
                    'termcolor==1.1.0',
                    'junit-xml==1.3',
                    'xmlrunner==1.7.7',
                    'ussclicore==1.0.9']

test_requires=['mock']


class CleanCommand(Command):
    """Custom clean command to tidy up the project root."""
    user_options = []
    def initialize_options(self):
        pass
    def finalize_options(self):
        pass
    def run(self):
        os.system('rm -vrf '+ROOT_DIR+'/build '+ROOT_DIR+'/dist '+ROOT_DIR+'/*.pyc '+ROOT_DIR+'/*.egg-info')
        os.system('find '+ROOT_DIR+' -iname "*.pyc" -exec rm {} +')

setup (  

  install_requires=requires,
  tests_require = test_requires,
  
  # Fill in these to make your Egg ready for upload to
  # PyPI
  name = 'hammr',
  version = VERSION,
  description='Command-line tool for building conistent and repeatable machine images for multiple cloud platforms',
  long_description='command-line tool for building/publishing/migrating consistent machine images for virtual datacenters and cloud platforms',
  packages = find_packages(),
  author = 'Joris Bremond',
  author_email = 'joris.bremond@usharesoft.com',
  license="Apache License 2.0",
  url = 'http://hammr.io',
  classifiers=(
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'Natural Language :: English',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: POSIX',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
    ),
       
  # ... custom build command
    cmdclass={
        'clean': CleanCommand,
    },

  #long_description= 'Long description of the package',
  scripts = ['bin/hammr', 'bin/hammr.bat'],
  
)



# Copyright 2007-2017 UShareSoft SAS, All rights reserved
#
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from unittest import TestCase

from hammr.utils.publish_utils import *


class TestPublishK5(TestCase):
    def test_publish_k5_should_return_publish_image_when_valid_entries(self):
        # given
        builder = self.build_builder("testName", "testDomain", "testProject", "testRegion")

        # when
        pimage = publish_k5vmdk(builder)

        # then
        self.assertEqual(pimage.displayName, builder["displayName"])
        self.assertEqual(pimage.keystoneDomain, builder["domain"])
        self.assertEqual(pimage.keystoneProject, builder["project"])
        self.assertEqual(pimage.publishLocation, builder["region"])


    def test_publish_k5_should_return_none_when_missing_name(self):
        # given
        builder = self.build_builder(None, "testDomain", "testProject", "testRegion")

        # when
        pimage = publish_k5vmdk(builder)

        # then
        self.assertIsNone(pimage)


    def test_publish_k5_should_return_none_when_missing_domain(self):
        # given
        builder = self.build_builder("testName", None, "testProject", "testRegion")

        # when
        pimage = publish_k5vmdk(builder)

        # then
        self.assertIsNone(pimage)


    def test_publish_k5_should_return_none_when_missing_project(self):
        # given
        builder = self.build_builder("testName", "testDomain", None, "testRegion")

        # when
        pimage = publish_k5vmdk(builder)

        # then
        self.assertIsNone(pimage)


    def test_publish_k5_should_return_none_when_missing_region(self):
        # given
        builder = self.build_builder("testName", "testDomain", "testProject", None)

        # when
        pimage = publish_k5vmdk(builder)

        # then
        self.assertIsNone(pimage)


    def build_builder(self, name, domain, project, region):
        builder = {}
        if name is not None: builder["displayName"] = name
        if domain is not None: builder["domain"] = domain
        if project is not None: builder["project"] = project
        if region is not None: builder["region"] = region
        return builder


class TestPublishDocker(TestCase):
    def test_publish_docker_should_return_publish_image_when_valid_entries(self):
        # given
        builder = self.build_builder("testNamespace", "testRepositoryName", "testTagName")

        # when
        pimage = publish_docker(builder)

        # then
        self.assertEqual(pimage.namespace, builder["namespace"])
        self.assertEqual(pimage.repositoryName, builder["repositoryName"])
        self.assertEqual(pimage.tagName, builder["tagName"])

    def test_publish_docker_should_return_none_when_missing_namespace(self):
        # given
        builder = self.build_builder(None, "testRepositoryName", "testTagName")

        # when
        pimage = publish_docker(builder)

        # then
        self.assertIsNone(pimage)

    def test_publish_docker_should_return_none_when_missing_repository_name(self):
        # given
        builder = self.build_builder("testNamespace", None, "testTagName")

        # when
        pimage = publish_docker(builder)

        # then
        self.assertIsNone(pimage)

    def test_publish_docker_should_return_none_when_missing_tag_name(self):
        # given
        builder = self.build_builder("testNamespace", "testRepositoryName", None)

        # when
        pimage = publish_docker(builder)

        # then
        self.assertIsNone(pimage)

    def build_builder(self, namespace, repository_name, tag_name):
        builder = {}
        if namespace is not None: builder["namespace"] = namespace
        if repository_name is not None: builder["repositoryName"] = repository_name
        if tag_name is not None: builder["tagName"] = tag_name
        return builder
